<?php

// *******************************************************************************
// **************       Store information about web page access      *************
// **************             AyMINE - WEB Module, example           *************
// **************                     (C) 2022, AyMINE               *************
// *******************************************************************************



// if (!isset($_ay_Path)) $_ay_Path = isset($_SERVER['DOCUMENT_ROOT']) ? $_SERVER['DOCUMENT_ROOT'] : dirname(getcwd(), 2);
// require_once $_ay_Path . '/frm/BE/framework.php';

abstract class ayWebServices
{
   // Access code from aymine settings
   public const accessCode='<code from aymine>';
   // Message identification - code from AyMINE settings
   public const msContactIdentification = '<service id from aymine>';
   public const userAgent = '<name used as a contact identification>';   

   /** Method that stores information about page access to the AyMINE access protocol
    * 
    */
   public static function storePageInfo(string $sourcePage, string|null $opCode = null)
   {
      
      // Required fields
      $qArray = [
         'accessCode' => ayWebServices::accessCode,
         'sourceIP' => $_SERVER['REMOTE_ADDR'],
         'sourcePage' => $sourcePage
      ];
      // Optional fields
      if (isset($opCode) && strlen($opCode) == 2) $qArray['operation'] = $opCode;

      $curl = curl_init('https://www.aymine.com/web/SVC/storePageAccess');

      // User verbose login in case of connection truoubles:
      // $debugCurl = fopen("c:\LocalCopy\curlDebug.txt", 'w');
      // see https://curl.se/libcurl/c/curl_easy_setopt.html for documenation of attributes
      curl_setopt_array($curl, array(
         CURLOPT_USERAGENT => ayWebServices::userAgent,     // who am i 
         CURLOPT_ENCODING => '',
         CURLOPT_MAXREDIRS => 10,
         CURLOPT_TIMEOUT_MS => 500,  // 500ms, see more at: https://curl.se/libcurl/c/CURLOPT_TIMEOUT_MS.html
         CURLOPT_FOLLOWLOCATION => true,
         CURLOPT_POST => true,
         CURLOPT_POSTFIELDS => $qArray,
         // CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_NONE,
         CURLOPT_CUSTOMREQUEST => 'POST'
         // , CURLOPT_VERBOSE        => 1,
         // CURLOPT_STDERR => $debugCurl
      ));

      curl_exec($curl);  // hopefully structure with required data
      $lastErr = curl_getinfo($curl, CURLINFO_RESPONSE_CODE);  // value from responce_code
      if ($lastErr !== 200) error_log(__METHOD__ . " Communication with storage server failed, error code: $lastErr");
   }

   /** Sends message with values from attributes 
    * @param string $name, - name from the web; should be filled
    * @param string $email, - email from web form
    * @param string $message - user message (should be filled, no optional)
    *  - user description in case that user only click a selection (like gets a newsletter)
    * @param string|null $msgIdentification - optional alternative identification of the message - supports creating differenct task
   */
   public static function sendMessageFromWeb(string $name, string $email, string $message, string|null $msgIdentification = null) : bool {

      $name = trim($name);
      $email = trim($email);
      $error = '';
      if (!isset($msgIdentification)) $msgIdentification = ayWebServices::msContactIdentification;

      // Check that email has valid structure
      if (!preg_match(
         "/^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$/i",
         $email
      )) {
         $error .= "Message rejected by by client: Invalid email address " . $email;
      }

      // Reject emails that contains web link in the message - most typical spam content
      if (preg_match('/<\s?[^\>]*\/?\s?>/', $message) ||  preg_match('/http|https|href|<a/', $message)) {
         $error .= " Error: Spam detected";
      }

      // Reject addresses that are not used by useres
      if (preg_match('/(\.biz$)|(\.icu$)/', $email))
         $error .= "  Error: spam domain";

      if (empty($error)) {

         $qArray = [
            'accessCode' => ayWebServices::accessCode,
            'msgIdentification' =>  $msgIdentification,
            'sourceIP' => $_SERVER['REMOTE_ADDR'],
            'sourcePage' => $_GET['sourcePage'] ?? $_POST['sourcePage'] ?? 'no location',
            'name' => $name,
            'email' => $email,
            "message" => $message
         ];

         $curl = curl_init('https://www.aymine.com/web/SVC/webInputMsg');

         curl_setopt_array($curl, array(
            CURLOPT_USERAGENT => ayWebServices::userAgent,     // who am i 
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => $qArray,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_RETURNTRANSFER => true   // necessary to get some resosne
         ));

         curl_exec($curl);  // hopefully structure with required data
         $lastErr = curl_getinfo($curl, CURLINFO_RESPONSE_CODE);  // value from responce_code
         if ($lastErr !== 200) {
            error_log("Message not send -> error / rejected by server; Error message: $lastErr for requester $email");   
            return false;
         }
         return true;
      } else {
         error_log("Message error not send -> spam  $email Reject resons: $error");
         return false;
      }
   }


   /** Creates message from web form with recors name / email / message 
    * @return bool - true - message send; false - message not send
   */
   public static function messageFromWeb() : bool
   {
      $errors = '';
      $myemail = 'stepan@nadrchal.eu'; //<-----Put Your email address here.
      if (
         empty($_POST['name'])  ||
         empty($_POST['email']) ||
         empty($_POST['message'])
      ) {
         $errors .= "\n Error: all fields are required";
      }

      $lerr = AyWebServices::sendMessageFromWeb($_POST['name'], $_POST['email'], $_POST['message'], $_POST['msgInformation'] ?? null);
      return $lerr;
   }
}

// AyWebServices::storePageInfo($_SERVER['PHP_SELF']);
// AyWebServices::messageFromWeb();
